import cq.checker
import cq.utils


class RuffChecker(cq.checker.Checker):

	NAME = 'ruff'
	DESCRIPTION = 'Run ruff linter on given modules.'
	HELP_LINE = 'use `# noqa: <rule-id>` for disabling ruff on particular line'

	def run(self, modules: list[str]) -> cq.checker.CheckerResult:
		options = [
			'check',
		]
		stdout, return_code = cq.utils.run_external_checker('ruff', options + modules)

		output: list[cq.checker.ResultLine] = []

		for output_line in stdout.split('\n'):
			if not output_line:
				continue

			# Ruff has a nice enough output, no need to split into file/line (there is also char pos)
			output.append(cq.checker.ResultLine(file = None, line = None, message = output_line, is_error = True))

		return cq.checker.CheckerResult(
			checker_name = self.NAME,
			help_line = self.HELP_LINE,
			return_code = return_code,
			output = output,
		)



from typing import TYPE_CHECKING
import os

import pytest

import cq.main


if TYPE_CHECKING:
	# This is importable only when it is run through pytest
	import click.testing.CliRunner  # noqa # pylint: disable=import-error


@pytest.fixture
def cli_runner() -> 'click.testing.CliRunner':
	from click.testing import CliRunner

	return CliRunner()


def test_default_modules(cli_runner: 'click.testing.CliRunner') -> None:
	'''
	Cannot use tmpdir fixture - we need to mock cwd
	'''
	with cli_runner.isolated_filesystem():
		with open('test.py', 'w', encoding = 'utf-8') as f:
			f.write('myVar = True\n')

		result = cli_runner.invoke(cq.main.main, [])
		assert result.exit_code == 0
		assert result.output == '✓\n'


def test_result_output(cli_runner: 'click.testing.CliRunner') -> None:
	with cli_runner.isolated_filesystem():
		with open('test.py', 'w', encoding = 'utf-8') as f:
			f.write('')

		result = cli_runner.invoke(cq.main.main, [os.path.abspath(f.name)])
		assert result.exit_code == 0
		assert result.output == '✓\n'



from typing import List

import pytest

import cq.checkers.regex


def _check_grammar_nazi(s: str) -> List[str]:
	return list(cq.checkers.regex.check_line(s, s, cq.checkers.regex.GrammarNaziChecker.RULES))


def test_exists() -> None:
	assert _check_grammar_nazi("Yo dawg, dis is Queen's English!") == []
	assert _check_grammar_nazi('Trade does NOT exists') == [  # grammar_checker: disable = 3rd-person-singular
		'Use "does not exist", not "does not exists". '  # grammar_checker: disable = 3rd-person-singular
		'The "does" verb already is in third person singular.'
	]
	assert _check_grammar_nazi('Trade may not exists') == [  # grammar_checker: disable = 3rd-person-singular
		'Use "may not exist", not "may not exists". '  # grammar_checker: disable = 3rd-person-singular
		'The "may" verb already is in third person singular.'
	]


def test_happend() -> None:  # grammar_checker: disable = happend
	assert _check_grammar_nazi('This can happend only ...') == [  # grammar_checker: disable = happend
		'"happend" is not a word. '  # grammar_checker: disable = happend
		'You either mean "happen" (present tense) or "happened" (past tense).'
	]


def test_it_self() -> None:
	assert _check_grammar_nazi('await self.x()') == []


def test_single() -> None:
	assert cq.checkers.regex.clobber_string_literals("print('hi')") == "print('..')"


def test_double() -> None:
	assert cq.checkers.regex.clobber_string_literals('print("hi")') == 'print("..")'


def test_escape_single() -> None:
	assert cq.checkers.regex.clobber_string_literals(''' q = '\\'' ''') == " q = '..' "


def test_escape_double() -> None:
	assert cq.checkers.regex.clobber_string_literals(''' q = "\\"" ''') == ' q = ".." '


def _check_dumb_style_checker(s: str) -> List[str]:
	return list(cq.checkers.regex.check_line(s, s, cq.checkers.regex.DumbStyleChecker.RULES))


@pytest.mark.parametrize(
	'source',
	(
		'a = 1',
		'a == 1',
		'a >= 1',
		'a <= 1',
		'a != 1',
		'a += 1',
		'a := 1',
		'a =',
	),
)
def test_spaces_around_equals_sign_negative(source: str) -> None:
	assert _check_dumb_style_checker(source) == []


@pytest.mark.parametrize(
	'source',
	(
		'print_this("hi", end="!")',
		'a=1',
		'a =1',
		'a= 1',
	),
)
def test_spaces_around_equals_sign_positive(source: str) -> None:
	result = _check_dumb_style_checker(source)
	assert len(result) == 1
	assert result[0].startswith('Put exactly one space before and after `=`')


@pytest.mark.parametrize(
	'source, operator',
	(
		('a!=1', '!='),
		('a !=1', '!='),
		('a>=1', '>='),
		('a== 1', '=='),
	),
)
def test_spaces_around_other_operators_positive(source: str, operator: str) -> None:
	result = _check_dumb_style_checker(source)
	assert len(result) == 1
	assert result[0].startswith(f'Put exactly one space before and after `{operator}`')


@pytest.mark.parametrize(
	'source',
	(
		'variable1 = ...',
		'variable2 = ...',
		'variable4 = ...',
		# These lines are ignore as an exception in regex
		'OAuth2Authenticator',
		'oauth2client.client',
	),
)
def test_number_in_names_negative(source: str) -> None:
	assert _check_dumb_style_checker(source) == []


@pytest.mark.parametrize(
	'source, number',
	(
		('def convert2string() -> str:', '2'),
		('def test4missing_rules() -> None:', '4'),
		('convert2price = []', '2'),
	),
)
def test_number_in_names_positive(source: str, number: str) -> None:
	result = _check_dumb_style_checker(source)
	assert len(result) == 1

	verb = 'to' if number == '2' else 'for'
	assert result[0].startswith(f'Do not spell `{verb}` as the digit `{number}`')


@pytest.mark.parametrize(
	'source',
	(
		'some_print(some_stuff)',
		'print(with_disable) # dumb_style_checker:disable = print-statement',
	),
)
def test_print_statement_negative(source: str) -> None:
	assert _check_dumb_style_checker(source) == []


@pytest.mark.parametrize(
	'source',
	(
		' print(some_stuff)',
		' print("other stuff")',
	),
)
def test_print_statement_positive(source: str) -> None:
	result = _check_dumb_style_checker(source)
	assert len(result) == 1
	assert result[0].startswith(
		'It seems that you have forgotten about a print statement. '
		'If this is intentional, disable this check by # dumb_style_checker:disable = print-statement'
	)


def test_operators_at_line_start() -> None:
	source = '''
assert (
	{
		'key': 42,
	}
	== test_func()
)
	'''
	assert _check_dumb_style_checker(source) == []



import pathlib
import tempfile

import cq.checkers.isort as isort


def test_isort_checker() -> None:
	checker = isort.IsortChecker()
	test_file = pathlib.Path(__file__).parent / 'data/isort.txt'
	with tempfile.TemporaryDirectory() as temp_dir:
		temp_path = pathlib.Path(temp_dir) / 'test.py'
		temp_path.write_text(test_file.read_text(encoding = 'utf-8'), encoding = 'utf-8')
		actual_result = checker.run([str(temp_path.resolve())])
		assert actual_result.return_code == 0
		assert len(actual_result.output) == 1
		assert actual_result.output[0].message == 'Imports are incorrectly sorted and/or formatted.'
		assert not actual_result.output[0].is_error



from typing import Any, Tuple
import sys

import cq.checker
import cq.checkers.safety
import cq.utils


NO_CONNECTION_ERROR = 'socket.gaierror: [Errno -3] Temporary failure in name resolution'


def pip_helper(package: str, uninstall: bool = False) -> Tuple[str, int]:
	options = ['-m', 'pip', 'uninstall', '-y', package] if uninstall else ['-m', 'pip', 'install', package]

	return cq.utils.run_external_checker(
		sys.executable,
		options,
	)


def test_runner(monkeypatch: Any) -> None:
	# first make sure insecure-package is not currently installed
	_, return_code = pip_helper('insecure-package', uninstall = True)
	assert return_code == 0

	checker = cq.checkers.safety.SafetyChecker()

	secure_result = checker.run([])
	assert secure_result.return_code == 0
	assert secure_result.output == []

	# install insecure-package
	_, return_code = pip_helper('insecure-package')
	assert return_code == 0

	insecure_result = checker.run([])
	assert insecure_result.return_code == 64
	assert insecure_result.output == [
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = 'package: insecure-package, affected: <0.2.0, installed: 0.1.0',
			is_error = True,
		)
	]

	# connection error case
	monkeypatch.setattr(cq.utils, 'run_external_checker', lambda cmd, options: (NO_CONNECTION_ERROR, 1))
	connection_error_result = checker.run([])

	assert connection_error_result.return_code == 0
	assert connection_error_result.output == [
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = cq.checkers.safety.SAFETY_CONNECTION_WARNING,
			is_error = False,
		)
	]

	# generic error case
	monkeypatch.setattr(cq.utils, 'run_external_checker', lambda cmd, options: ('', 7))
	generic_error_result = checker.run([])
	assert generic_error_result.return_code == 7
	assert generic_error_result.output == [
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = cq.checkers.safety.SAFETY_OTHER_ERROR,
			is_error = True,
		)
	]

	# decode error case
	monkeypatch.setattr(cq.utils, 'run_external_checker', lambda cmd, options: ('this is not json', 64))
	decode_error_result = checker.run([])
	assert decode_error_result.return_code == 64
	assert decode_error_result.output == [
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = cq.checkers.safety.SAFETY_DECODE_ERROR,
			is_error = True,
		)
	]

	monkeypatch.undo()

	# finally uninstall insecure-package
	_, return_code = pip_helper('insecure-package', uninstall = True)
	assert return_code == 0



import pathlib

import py
import pytest

import cq.checkers.init as checker


def make_module(path: pathlib.Path, module_name: str = 'module', include_init: bool = False) -> pathlib.Path:
	module = path / module_name
	module.mkdir()
	(module / 'main.py').touch()
	if include_init:
		(module / '__init__.py').touch()
	return module


@pytest.mark.parametrize('include_init', [True, False])
def test_simple_module(tmpdir: py.path.local, include_init: bool) -> None:
	''' Top-level module can be with or without __init__.py '''
	tmp = pathlib.Path(tmpdir)
	module = make_module(tmp, include_init = include_init)
	errors = checker.find_missing_inits(module)
	assert list(errors) == []


def test_module_with_subdirectory(tmpdir: py.path.local) -> None:
	''' Should ignore subdirectories as long as they don't have *.py file in them '''
	tmp = pathlib.Path(tmpdir)
	module = make_module(tmp)
	subdirectory = module / 'subdir'
	subdirectory.mkdir()
	(subdirectory / 'hello.txt').touch()
	errors = checker.find_missing_inits(module)
	assert list(errors) == []


@pytest.mark.parametrize('include_init', [True, False])
@pytest.mark.parametrize('submodule_name', ['submodule', '.hidden'])
def test_module_with_submodule(tmpdir: py.path.local, include_init: bool, submodule_name: str) -> None:
	''' Submodules must have __init__.py, should ignore hidden directories '''
	tmp = pathlib.Path(tmpdir)
	module = make_module(tmp)
	submodule = make_module(module, module_name = submodule_name, include_init = include_init)
	errors = checker.find_missing_inits(module)
	if include_init or submodule_name == '.hidden':
		assert list(errors) == []
	else:
		assert list(errors) == [submodule]


def test_deep_nested_submodule(tmpdir: py.path.local) -> None:
	''' Should recursively check submodules (even those without __init__.py) '''
	tmp = pathlib.Path(tmpdir)
	module = make_module(tmp)
	level_1_1 = make_module(module, module_name = 'level_1_1', include_init = True)
	level_1_2 = make_module(module, module_name = 'level_1_2', include_init = False)
	level_2_1 = make_module(level_1_1, module_name = 'level_2_1', include_init = True)
	level_2_2 = make_module(level_1_1, module_name = 'level_2_2', include_init = False)
	level_2_3 = make_module(level_1_2, module_name = 'level_2_3', include_init = False)
	level_3_1 = make_module(level_2_1, module_name = 'level_3_1', include_init = False)
	errors = checker.find_missing_inits(module)
	assert set(errors) == {level_1_2, level_2_2, level_2_3, level_3_1}



from typing import Any

import cq.checker
import cq.checkers.mypy
import cq.utils


OUTPUT_STRING = '''file.py:3: error: Need type annotation for 'd'
file.py: This file is completely wrong
and I want this line to be in output also'''


def test_runner(monkeypatch: Any) -> None:
	monkeypatch.setattr(cq.utils, 'run_external_checker', lambda cmd, options: (OUTPUT_STRING, 1))

	checker = cq.checkers.mypy.MypyChecker()

	actual_result = checker.run([])

	assert actual_result.output == [
		cq.checker.ResultLine(
			file = 'file.py',
			line = 3,
			message = "error: Need type annotation for 'd'",
			is_error = True,
		),
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = 'file.py: This file is completely wrong',
			is_error = True,
		),
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = 'and I want this line to be in output also',
			is_error = True,
		),
	]



# pylint: disable = protected-access
import packaging.version
import pytest
import requirements

import cq.checkers.requirements


version = packaging.version.parse
CHECKER = cq.checkers.requirements.RequirementsSetupCompatiblityChecker


# Mypy error: Untyped decorator makes function "cli_runner" untyped
@pytest.fixture()
def checker() -> CHECKER:
	return CHECKER()


# requirements should be subset of setup
# Mypy error: Untyped decorator makes function "cli_runner" untyped
@pytest.mark.parametrize(
	'requirements, setup, expected_result',
	[
		('a==1.0.0', '', "setup.py: does not contain requirement 'a' that is in requirements.txt"),
		('a==1.2.3', 'a>=1.0.0,<2.0.0', ''),
		(
			'a==2.2.3',
			'a>=1.0.0,<2.0.0',
			"setup.py: requirement 'a' version in requirements.txt (==2.2.3) "
			'does not lie in version of setup.py (>=1.0.0,<2.0.0)',
		),
		('a<5.0.0', 'a<10.0.0', ''),
		('a>2.0.0', 'a>1.0.0', ''),
		('a>1.0.0,<2.0.0', 'a>1.0.0,<2.0.0', ''),
		(
			'a>2.2.0',
			'a<2.0.0',
			"setup.py: requirement 'a' version in requirements.txt (>2.2.0,<=MAX) "
			'does not lie in version of setup.py (>=MIN,<2.0.0)',
		),
		(
			'a>2018.1,<2019.2',
			'a>2019.4',
			"setup.py: requirement 'a' version in requirements.txt (>2018.1,<2019.2) "
			'does not lie in version of setup.py (>2019.4,<=MAX)',
		),
		('a>2018.1,<2019.2', 'a>2017.4', ''),
		('a>2018.1', 'a>2017.4', ''),
		('a<2018.1', 'a<2019.4', ''),
		('a==8', 'a>5,<10', ''),
		('a>=8,<10', 'a>5,<10', ''),
		(
			'a>8',
			'a>5,<10',
			"setup.py: requirement 'a' version in requirements.txt (>8,<=MAX) "
			'does not lie in version of setup.py (>5,<10)',
		),
		(
			'a>12',
			'a>5,<10',
			"setup.py: requirement 'a' version in requirements.txt (>12,<=MAX) "
			'does not lie in version of setup.py (>5,<10)',
		),
		(
			'a>12',
			'a<10',
			"setup.py: requirement 'a' version in requirements.txt (>12,<=MAX) "
			'does not lie in version of setup.py (>=MIN,<10)',
		),
	],
)
def test_requirements_setup_compatibility(
	requirements: str,
	setup: str,
	expected_result: str,
	checker: CHECKER,
) -> None:
	result = checker._check_requirements_setup_compatibility(requirements, setup)
	if expected_result:
		assert result[0].message == expected_result
	else:
		assert not result


def test_version_range_from_specs() -> None:
	specs_str = 'a>=1.0.0,<2.0.0'
	specs = list(requirements.parse(specs_str))[0].specs
	version_range = cq.checkers.requirements.VersionRange.from_specs(specs)  # type: ignore[arg-type]
	assert version_range.lower == version('1.0.0')
	assert version_range.upper == version('2.0.0')
	assert version_range.lower_inclusive
	assert not version_range.upper_inclusive


def test_version_range_from_specs_singleton() -> None:
	specs_str = 'a==1.0.0'
	specs = list(requirements.parse(specs_str))[0].specs
	version_range = cq.checkers.requirements.VersionRange.from_specs(specs)  # type: ignore[arg-type]
	assert version_range.lower == version('1.0.0')
	assert version_range.upper == version('1.0.0')
	assert version_range.lower_inclusive
	assert version_range.upper_inclusive


def test_version_range_subset() -> None:
	vr1 = cq.checkers.requirements.VersionRange(
		version('1.0.0'),
		version('1.1.0'),
		lower_inclusive = True,
		upper_inclusive = False,
	)
	vr2 = cq.checkers.requirements.VersionRange(
		version('1.0.0'),
		version('2.0.0'),
		lower_inclusive = True,
		upper_inclusive = False,
	)

	assert vr1.issubset(vr2)
	assert not vr2.issubset(vr1)

	# vr1 lower is outside of vr2 lower
	vr1.lower = version('0.1.0')
	assert not vr1.issubset(vr2)

	# vr2 lower is not inclusive anymore
	vr1.lower = version('1.0.0')
	vr2.lower_inclusive = False
	assert not vr1.issubset(vr2)



import os

import py

import cq.utils


def test_simple_module(tmpdir: py.path.local) -> None:
	f = tmpdir.mkdir('module').join('__init__.py')
	f.write('')
	module_name = os.path.dirname(str(f))
	gen = cq.utils.get_path_generator([module_name])
	assert list(gen) == [module_name]


def test_python_file(tmpdir: py.path.local) -> None:
	f = tmpdir.mkdir('module').join('test.py')
	f.write('')
	module_name = str(f)
	gen = cq.utils.get_path_generator([module_name])
	assert list(gen) == [module_name]


def test_directory_of_python_files(tmpdir: py.path.local) -> None:
	f = tmpdir.mkdir('module').join('test.py')
	f.write('')
	g = tmpdir.join('module').join('test2.py')
	g.write('')
	path1 = str(f)
	path2 = str(g)
	gen = cq.utils.get_path_generator([os.path.dirname(path1)])
	assert set(gen) == {path1, path2}


def test_combination(tmpdir: py.path.local) -> None:
	f = tmpdir.mkdir('module').join('test.py')
	f.write('')
	g = tmpdir.join('module').mkdir('pymodule').join('__init__.py')
	g.write('')

	path1 = str(f)
	dir1 = os.path.dirname(path1)
	path2 = str(g)
	dir2 = os.path.dirname(path2)
	gen = cq.utils.get_path_generator([dir1, dir2])
	assert set(gen) == {path1, dir2}


def test_empty(tmpdir: py.path.local) -> None:
	f = tmpdir.mkdir('module').join('empty_file')
	f.write('')

	path = str(f)
	gen = cq.utils.get_path_generator([path])
	assert list(gen) == []


def test_hidden_file(tmpdir: py.path.local) -> None:
	f = tmpdir.mkdir('module').join('.hidden.py')
	f.write('')

	path = str(f)
	gen = cq.utils.get_path_generator([path])
	assert list(gen) == []


def test_hidden_dir(tmpdir: py.path.local) -> None:
	f = tmpdir.mkdir('.module').mkdir('sub').join('hidden.py')
	f.write('')

	path = str(f)
	gen = cq.utils.get_path_generator([path])
	assert list(gen) == []



from typing import Any

import cq.checker
import cq.checkers.pylint
import cq.utils


OUTPUT_STRING = '''************* Module test_volume_monitor
dev.py:2: [E0401(import-error), ] Unable to import 'decouple'
file.py: This file is completely wrong
and I want this line to be in output also
'''


def test_runner(monkeypatch: Any) -> None:
	monkeypatch.setattr(cq.utils, 'run_external_checker', lambda cmd, options, stderr: (OUTPUT_STRING, 1))

	checker = cq.checkers.pylint.PylintChecker()

	actual_result = checker.run([])

	assert actual_result.output == [
		cq.checker.ResultLine(
			file = 'dev.py',
			line = 2,
			message = "[E0401(import-error), ] Unable to import 'decouple'",
			is_error = True,
		),
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = 'file.py: This file is completely wrong',
			is_error = False,
		),
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = 'and I want this line to be in output also',
			is_error = False,
		),
	]



from typing import Any

import cq.checker
import cq.checkers.ruff
import cq.utils


OUTPUT_STRING = '''file.py:3:5 E721 Do not compare types, use `isinstance()`
file.py: This file is completely wrong
and I want this line to be in output also'''


def test_runner(monkeypatch: Any) -> None:
	monkeypatch.setattr(cq.utils, 'run_external_checker', lambda cmd, options: (OUTPUT_STRING, 1))

	checker = cq.checkers.ruff.RuffChecker()

	actual_result = checker.run([])
	assert actual_result.output == [
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = 'file.py:3:5 E721 Do not compare types, use `isinstance()`',
			is_error = True,
		),
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = 'file.py: This file is completely wrong',
			is_error = True,
		),
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = 'and I want this line to be in output also',
			is_error = True,
		),
	]



from typing import Any
import pathlib

import pytest

import cq.checker
import cq.checkers.orange as orange
import cq.utils


@pytest.mark.parametrize('filename', ('not-formatted.txt', 'formatted.txt'))
def test_runner_with_orange_installed(filename: str) -> None:
	checker = orange.OrangeChecker()
	test_file = pathlib.Path(__file__).parent / f'data/{filename}'
	actual_result = checker.run([str(test_file.resolve())])
	if filename == 'not-formatted.txt':
		assert actual_result.return_code == 1
		assert actual_result.output == [
			cq.checker.ResultLine(
				file = None,
				line = None,
				message = '1 file would be reformatted.',
				is_error = True,
			)
		]
	else:
		assert actual_result.return_code == 0
		assert actual_result.output == []


def test_runner_without_orange_insalled(monkeypatch: Any) -> None:
	def _orange_not_found(cmd: str, options: Any) -> None:
		raise FileNotFoundError

	monkeypatch.setattr(cq.utils, 'run_external_checker', _orange_not_found)
	checker = orange.OrangeChecker()
	test_file = pathlib.Path(__file__).parent / 'data/not-formatted.txt'
	actual_result = checker.run([str(test_file.resolve())])
	assert actual_result.return_code == 0
	assert actual_result.output == [
		cq.checker.ResultLine(
			file = None,
			line = None,
			message = orange.OrangeChecker.ORANGE_NOT_INSTALLED,
			is_error = False,
		)
	]



import pathlib
import tempfile

import cq.fixers.isort_fixer as isort


def test_isort_fixer() -> None:
	fixer = isort.IsortFixer()
	test_path = pathlib.Path(__file__).parent / 'data/isort-test.txt'
	result_path = pathlib.Path(__file__).parent / 'data/isort-result.txt'
	with tempfile.TemporaryDirectory() as temp_dir:
		temp_path = pathlib.Path(temp_dir) / 'test.py'
		temp_path.write_text(test_path.read_text(encoding = 'utf-8'), encoding = 'utf-8')
		fixer.run([str(temp_path)])
		assert temp_path.read_text(encoding = 'utf-8') == result_path.read_text(encoding = 'utf-8')



from typing import Any, Dict, List
import dataclasses
import json

import cq.checker
import cq.utils


SAFETY_CONNECTION_WARNING = 'warning: safety checker cannot download vulnerability database, skipping'
SAFETY_OTHER_ERROR = 'safety checker encountered an error'
SAFETY_DECODE_ERROR = 'error when decoding safety checker output'


@dataclasses.dataclass
class SafetyReportEntry:
	vulnerability_id: str
	package_name: str
	vulnerable_spec: str
	analyzed_version: str
	advisory: str


class SafetyChecker(cq.checker.Checker):
	NAME = 'safety'
	DESCRIPTION = 'Checks installed dependencies for known security vulnerabilities.'

	def run(self, modules: List[str]) -> cq.checker.CheckerResult:
		stdout, return_code = cq.utils.run_external_checker(self.NAME, ['check', '--json'])

		# 0 is returned when safety does not find a vulnerability
		# 64 is returned when safety finds a vulnerability
		# everything else should be considered as warning or error in cq
		if return_code not in (0, 64):
			if return_code == 1 and any(keyword in stdout for keyword in ('NewConnectionError', 'socket.gaierror')):
				# we never want to fail cq because of connection error of safety checker
				return_code = 0
				output = [
					cq.checker.ResultLine(
						file = None,
						line = None,
						message = SAFETY_CONNECTION_WARNING,
						is_error = False,
					)
				]
			else:
				# if the reason for safety checker error is not connection error, let it fail cq
				output = [
					cq.checker.ResultLine(
						file = None,
						line = None,
						message = SAFETY_OTHER_ERROR,
						is_error = True,
					)
				]
		else:
			try:
				data = json.loads(stdout[stdout.find('{') :])
				output = [SafetyChecker.entry_to_result_line(entry) for entry in data['vulnerabilities']]
			except (json.decoder.JSONDecodeError, TypeError):
				output = [
					cq.checker.ResultLine(
						file = None,
						line = None,
						message = SAFETY_DECODE_ERROR,
						is_error = True,
					)
				]

		return cq.checker.CheckerResult(
			checker_name = self.NAME,
			help_line = None,
			return_code = return_code,
			output = output,
		)

	@staticmethod
	def entry_to_result_line(output_entry: Dict[str, Any]) -> cq.checker.ResultLine:
		safety_report_entry = SafetyReportEntry(
			**{field.name: output_entry[field.name] for field in dataclasses.fields(SafetyReportEntry)}
		)

		return cq.checker.ResultLine(
			file = None,
			line = None,
			message = f'package: {safety_report_entry.package_name}, '
			f'affected: {safety_report_entry.vulnerable_spec}, '
			f'installed: {safety_report_entry.analyzed_version}',
			is_error = True,
		)



from typing import Any, Tuple
import os

import pytest

import cq.checkers.git
import cq.utils


CHECKER = cq.checkers.git.BranchNameChecker


@pytest.fixture
def branch_name_checker() -> CHECKER:
	return CHECKER()


def test_correct_branch_name_env(monkeypatch: Any, branch_name_checker: CHECKER) -> None:
	monkeypatch.setattr(os, 'environ', {'CI_COMMIT_REF_NAME': 'feature/cool_new_feature'})

	result = branch_name_checker.run([])
	assert result.return_code == 0


def test_correct_branch_name_git(monkeypatch: Any, branch_name_checker: CHECKER) -> None:
	def runner(cmd: str, _options: Any) -> Tuple[str, int]:
		if cmd != 'git':
			assert False

		return ('master', 0)

	monkeypatch.setattr(cq.utils, 'run_external_checker', runner)

	result = branch_name_checker.run([])
	assert result.return_code == 0


def test_incorrect_branch_name_env(monkeypatch: Any, branch_name_checker: CHECKER) -> None:
	monkeypatch.setattr(os, 'environ', {'CI_COMMIT_REF_NAME': 'incorrectly_named_branch'})

	result = branch_name_checker.run([])
	assert result.return_code == 1
